@@ -2,7 +2,6 @@ from __future__ import division
 
 import unittest
 import os
-import six
 import re
 import shutil
 import time
@@ -10,11 +9,11 @@ import time
 from packaging.version import Version
 
 import overrides_hack
-from utils import create_sparse_tempfile, create_lio_device, delete_lio_device, fake_utils, fake_path, mount, umount, run_command, TestTags, tag_test
+from utils import create_sparse_tempfile, create_lio_device, delete_lio_device, fake_utils, fake_path, mount, umount, run_command, TestTags, tag_test, required_plugins
 
 import gi
-gi.require_version("BlockDev", "2.0")
-
+gi.require_version('GLib', '2.0')
+gi.require_version('BlockDev', '3.0')
 from gi.repository import GLib, BlockDev
 
 TEST_MNT = "/tmp/libblockdev_test_mnt"
@@ -23,8 +22,8 @@ def wipefs(device):
     os.system("wipefs -a %s > /dev/null" % device)
 
 
-class BtrfsTestCase(unittest.TestCase):
-
+@required_plugins(("btrfs",))
+class BtrfsTest(unittest.TestCase):
     requested_plugins = BlockDev.plugin_specs_from_names(("btrfs",))
 
     @classmethod
@@ -41,34 +40,33 @@ class BtrfsTestCase(unittest.TestCase):
         else:
             BlockDev.reinit(cls.requested_plugins, True, None)
 
-    def setUp(self):
-        self.addCleanup(self._clean_up)
-        self.dev_file = create_sparse_tempfile("btrfs_test", 1024**3)
-        self.dev_file2 = create_sparse_tempfile("btrfs_test", 1024**3)
-        try:
-            self.loop_dev = create_lio_device(self.dev_file)
-        except RuntimeError as e:
-            raise RuntimeError("Failed to setup loop device for testing: %s" % e)
-        try:
-            self.loop_dev2 = create_lio_device(self.dev_file2)
-        except RuntimeError as e:
-            raise RuntimeError("Failed to setup loop device for testing: %s" % e)
+class BtrfsPluginVersionCase(BtrfsTest):
+    @tag_test(TestTags.NOSTORAGE)
+    def test_plugin_version(self):
+        self.assertEqual(BlockDev.get_plugin_soname(BlockDev.Plugin.BTRFS), "libbd_btrfs.so.3")
 
-    def _clean_up(self):
+class BtrfsTestCase(BtrfsTest):
+
+    def setUp(self):
+        self.dev_file, self.loop_dev = self.setup_test_device()
+
+    def setup_test_device(self, dev_file_size=1024**3):
+        dev_file = create_sparse_tempfile("btrfs_test", dev_file_size)
+        try:
+            loop_dev = create_lio_device(dev_file)
+            self.addCleanup(self._clean_up, loop_dev, dev_file)
+        except RuntimeError as e:
+            raise RuntimeError("Failed to setup loop device for testing: %s" % e)
+        return dev_file, loop_dev
+
+    def _clean_up(self, loop_dev, dev_file):
         umount(TEST_MNT)
         try:
-            delete_lio_device(self.loop_dev)
+            delete_lio_device(loop_dev)
         except RuntimeError:
             # just move on, we can do no better here
             pass
-        os.unlink(self.dev_file)
-
-        try:
-            delete_lio_device(self.loop_dev2)
-        except RuntimeError:
-            # just move on, we can do no better here
-            pass
-        os.unlink(self.dev_file2)
+        os.unlink(dev_file)
 
     def _get_btrfs_version(self):
         _ret, out, _err = run_command("btrfs --version")
@@ -77,6 +75,12 @@ class BtrfsTestCase(unittest.TestCase):
             raise RuntimeError("Failed to determine btrfs version from: %s" % out)
         return Version(m.groups()[0])
 
+
+class BtrfsMultiTestCase(BtrfsTestCase):
+    def setUp(self):
+        self.dev_file, self.loop_dev = self.setup_test_device()
+        self.dev_file2, self.loop_dev2 = self.setup_test_device()
+
 class BtrfsTestCreateQuerySimple(BtrfsTestCase):
     @tag_test(TestTags.CORE)
     def test_create_and_query_volume(self):
@@ -117,7 +121,7 @@ class BtrfsTestCreateQueryLabel(BtrfsTestCase):
         self.assertEqual(len(devs), 1)
 
 
-class BtrfsTestCreateQueryTwoDevs(BtrfsTestCase):
+class BtrfsTestCreateQueryTwoDevs(BtrfsMultiTestCase):
     def test_create_and_query_volume_two_devs(self):
         """Verify that btrfs volume creation with two devices works"""
 
@@ -128,7 +132,7 @@ class BtrfsTestCreateQueryTwoDevs(BtrfsTestCase):
         devs = BlockDev.btrfs_list_devices(self.loop_dev)
         self.assertEqual(len(devs), 2)
 
-class BtrfsTestCreateQueryTwoDevsRaids(BtrfsTestCase):
+class BtrfsTestCreateQueryTwoDevsRaids(BtrfsMultiTestCase):
     def test_create_and_query_volume_two_devs(self):
         """Verify that btrfs volume creation with two devices and raid (meta)data works"""
 
@@ -160,7 +164,7 @@ class BtrfsTestCreateQueryTwoDevsRaids(BtrfsTestCase):
         devs = BlockDev.btrfs_list_devices(self.loop_dev)
         self.assertEqual(len(devs), 2)
 
-class BtrfsTestAddRemoveDevice(BtrfsTestCase):
+class BtrfsTestAddRemoveDevice(BtrfsMultiTestCase):
     def test_add_remove_device(self):
         """Verify that it is possible to add/remove device to a btrfs volume"""
 
@@ -189,10 +193,6 @@ class BtrfsTestCreateDeleteSubvolume(BtrfsTestCase):
     def test_create_delete_subvolume(self):
         """Verify that it is possible to create/delete subvolume"""
 
-        btrfs_version = self._get_btrfs_version()
-        if btrfs_version >= Version('4.13.2'):
-            self.skipTest('subvolumes list is broken with btrfs-progs v4.13.2')
-
         succ = BlockDev.btrfs_create_volume([self.loop_dev], "myShinyBtrfs", None, None, None)
         self.assertTrue(succ)
 
@@ -208,8 +208,10 @@ class BtrfsTestCreateDeleteSubvolume(BtrfsTestCase):
         self.assertEqual(len(subvols), 1)
 
         # already there
-        with self.assertRaises(GLib.GError):
-            BlockDev.btrfs_create_subvolume(TEST_MNT, "subvol1", None)
+        version = self._get_btrfs_version()
+        if version != Version("6.6.3"):
+            with self.assertRaises(GLib.GError):
+                BlockDev.btrfs_create_subvolume(TEST_MNT, "subvol1", None)
 
         succ = BlockDev.btrfs_delete_subvolume(TEST_MNT, "subvol1", None)
         self.assertTrue(succ)
@@ -218,8 +220,9 @@ class BtrfsTestCreateDeleteSubvolume(BtrfsTestCase):
         self.assertEqual(len(subvols), 0)
 
         # already removed
-        with self.assertRaises(GLib.GError):
-            BlockDev.btrfs_delete_subvolume(TEST_MNT, "subvol1", None)
+        if version != Version("6.6.3"):
+            with self.assertRaises(GLib.GError):
+                BlockDev.btrfs_delete_subvolume(TEST_MNT, "subvol1", None)
 
         succ = BlockDev.btrfs_create_subvolume(TEST_MNT, "subvol1", None)
         self.assertTrue(succ)
@@ -273,7 +276,7 @@ class BtrfsTestGetDefaultSubvolumeID(BtrfsTestCase):
         self.assertTrue(succ)
 
         # not mounted yet, should fail
-        with six.assertRaisesRegex(self, GLib.GError, r".*(can't|cannot) access.*"):
+        with self.assertRaisesRegex(GLib.GError, r".*(can't|cannot) access.*"):
             ret = BlockDev.btrfs_get_default_subvolume_id(TEST_MNT)
 
         mount(self.loop_dev, TEST_MNT)
@@ -311,7 +314,7 @@ class BtrfsTestSetDefaultSubvolumeID(BtrfsTestCase):
         ret = BlockDev.btrfs_get_default_subvolume_id(TEST_MNT)
         self.assertEqual(ret, 5)
 
-class BtrfsTestListDevices(BtrfsTestCase):
+class BtrfsTestListDevices(BtrfsMultiTestCase):
     @tag_test(TestTags.CORE)
     def test_list_devices(self):
         """Verify that it is possible to get info about devices"""
@@ -325,10 +328,10 @@ class BtrfsTestListDevices(BtrfsTestCase):
         self.assertEqual(devs[1].id, 2)
         self.assertEqual(devs[0].path, self.loop_dev)
         self.assertEqual(devs[1].path, self.loop_dev2)
-        self.assertTrue(devs[0].size >= 0)
-        self.assertTrue(devs[1].size >= 0)
-        self.assertTrue(devs[0].used >= 0)
-        self.assertTrue(devs[1].used >= 0)
+        self.assertGreaterEqual(devs[0].size, 0)
+        self.assertGreaterEqual(devs[1].size, 0)
+        self.assertGreaterEqual(devs[0].used, 0)
+        self.assertGreaterEqual(devs[1].used, 0)
 
 class BtrfsTestListSubvolumes(BtrfsTestCase):
     @tag_test(TestTags.CORE)
@@ -346,13 +349,52 @@ class BtrfsTestListSubvolumes(BtrfsTestCase):
         succ = BlockDev.btrfs_create_subvolume(TEST_MNT, "subvol1", None)
         self.assertTrue(succ)
 
+        succ = BlockDev.btrfs_create_subvolume(TEST_MNT, "subvol1/bar", None)
+        self.assertTrue(succ)
+
         subvols = BlockDev.btrfs_list_subvolumes(TEST_MNT, False)
-        self.assertEqual(len(subvols), 1)
+        self.assertEqual(len(subvols), 2)
         self.assertEqual(subvols[0].parent_id, 5)
         self.assertEqual(subvols[0].path, "subvol1")
+        self.assertEqual(subvols[1].path, "subvol1/bar")
+
+        # test also subvolumes with spaces in name
+        succ = BlockDev.btrfs_create_subvolume(TEST_MNT, "subvol with spaces", None)
+        self.assertTrue(succ)
+
+        subvols = BlockDev.btrfs_list_subvolumes(TEST_MNT, False)
+        self.assertTrue(any(subvol.path == "subvol with spaces" for subvol in subvols))
+
+    @tag_test(TestTags.CORE)
+    def test_list_subvolumes_different_mount(self):
+        """Verify that it is possible get to info about subvolumes with subvol= mount option"""
+
+        succ = BlockDev.btrfs_create_volume([self.loop_dev], "myShinyBtrfs", None, None, None)
+        self.assertTrue(succ)
+
+        mount(self.loop_dev, TEST_MNT)
+
+        succ = BlockDev.btrfs_create_subvolume(TEST_MNT, "one", None)
+        self.assertTrue(succ)
+        succ = BlockDev.btrfs_create_subvolume(TEST_MNT, "one/two", None)
+        self.assertTrue(succ)
+
+        os.system("mkdir -p mkdir -p %s/one/two/one/two" % (TEST_MNT))
+
+        succ = BlockDev.btrfs_create_subvolume(TEST_MNT, "one/two/one/two/three", None)
+        self.assertTrue(succ)
+
+        umount(TEST_MNT)
+        os.makedirs(TEST_MNT)
+        os.system("mount -o subvol=%s %s %s" % ("one/two", self.loop_dev, TEST_MNT))
+
+        subvols = BlockDev.btrfs_list_subvolumes(TEST_MNT, False)
+        self.assertEqual(len(subvols), 3)
+        self.assertEqual(subvols[0].path, "one")
+        self.assertEqual(subvols[1].path, "one/two")
+        self.assertEqual(subvols[2].path, "one/two/one/two/three")
 
 class BtrfsTestFilesystemInfo(BtrfsTestCase):
-    @tag_test(TestTags.CORE)
     def test_filesystem_info(self):
         """Verify that it is possible to get filesystem info"""
 
@@ -367,7 +409,7 @@ class BtrfsTestFilesystemInfo(BtrfsTestCase):
         self.assertEqual(info.label, label)
         self.assertTrue(info.uuid)
         self.assertEqual(info.num_devices, 1)
-        self.assertTrue(info.used >= 0)
+        self.assertGreaterEqual(info.used, 0)
 
 class BtrfsTestFilesystemInfoNoLabel(BtrfsTestCase):
     def test_filesystem_info(self):
@@ -382,7 +424,7 @@ class BtrfsTestFilesystemInfoNoLabel(BtrfsTestCase):
         self.assertEqual(info.label, str())
         self.assertTrue(info.uuid)
         self.assertEqual(info.num_devices, 1)
-        self.assertTrue(info.used >= 0)
+        self.assertGreaterEqual(info.used, 0)
 
 class BtrfsTestMkfs(BtrfsTestCase):
     @tag_test(TestTags.CORE)
@@ -470,34 +512,11 @@ class BtrfsTestChangeLabel(BtrfsTestCase):
         info = BlockDev.btrfs_filesystem_info(TEST_MNT)
         self.assertEqual(info.label, "newLabel")
 
-class BtrfsTooSmallTestCase (BtrfsTestCase):
+class BtrfsTooSmallTestCase (BtrfsMultiTestCase):
     def setUp(self):
-        self.addCleanup(self._clean_up)
-        self.dev_file = create_sparse_tempfile("btrfs_test", BlockDev.BTRFS_MIN_MEMBER_SIZE)
-        self.dev_file2 = create_sparse_tempfile("btrfs_test", BlockDev.BTRFS_MIN_MEMBER_SIZE//2)
-        try:
-            self.loop_dev = create_lio_device(self.dev_file)
-        except RuntimeError as e:
-            raise RuntimeError("Failed to setup loop device for testing: %s" % e)
-        try:
-            self.loop_dev2 = create_lio_device(self.dev_file2)
-        except RuntimeError as e:
-            raise RuntimeError("Failed to setup loop device for testing: %s" % e)
 
-    def _clean_up(self):
-        try:
-            delete_lio_device(self.loop_dev)
-        except RuntimeError:
-            # just move on, we can do no better here
-            pass
-        os.unlink(self.dev_file)
-
-        try:
-            delete_lio_device(self.loop_dev2)
-        except RuntimeError:
-            # just move on, we can do no better here
-            pass
-        os.unlink(self.dev_file2)
+        self.dev_file, self.loop_dev = self.setup_test_device(BlockDev.BTRFS_MIN_MEMBER_SIZE)
+        self.dev_file2, self.loop_dev2 = self.setup_test_device(BlockDev.BTRFS_MIN_MEMBER_SIZE//2)
 
     def test_create_too_small(self):
         """Verify that an attempt to create BTRFS on a too small device fails"""
@@ -507,34 +526,10 @@ class BtrfsTooSmallTestCase (BtrfsTestCase):
             BlockDev.btrfs_create_volume([self.loop_dev, self.loop_dev2],
                                          None, None, None)
 
-class BtrfsJustBigEnoughTestCase (BtrfsTestCase):
+class BtrfsJustBigEnoughTestCase (BtrfsMultiTestCase):
     def setUp(self):
-        self.addCleanup(self._clean_up)
-        self.dev_file = create_sparse_tempfile("btrfs_test", BlockDev.BTRFS_MIN_MEMBER_SIZE)
-        self.dev_file2 = create_sparse_tempfile("btrfs_test", BlockDev.BTRFS_MIN_MEMBER_SIZE)
-        try:
-            self.loop_dev = create_lio_device(self.dev_file)
-        except RuntimeError as e:
-            raise RuntimeError("Failed to setup loop device for testing: %s" % e)
-        try:
-            self.loop_dev2 = create_lio_device(self.dev_file2)
-        except RuntimeError as e:
-            raise RuntimeError("Failed to setup loop device for testing: %s" % e)
-
-    def _clean_up(self):
-        try:
-            delete_lio_device(self.loop_dev)
-        except RuntimeError:
-            # just move on, we can do no better here
-            pass
-        os.unlink(self.dev_file)
-
-        try:
-            delete_lio_device(self.loop_dev2)
-        except RuntimeError:
-            # just move on, we can do no better here
-            pass
-        os.unlink(self.dev_file2)
+        self.dev_file, self.loop_dev = self.setup_test_device(BlockDev.BTRFS_MIN_MEMBER_SIZE)
+        self.dev_file2, self.loop_dev2 = self.setup_test_device(BlockDev.BTRFS_MIN_MEMBER_SIZE)
 
     def test_create_just_enough(self):
         """Verify that creating BTRFS on a just big enough devices works"""
@@ -544,16 +539,14 @@ class BtrfsJustBigEnoughTestCase (BtrfsTestCase):
         self.assertTrue(succ)
 
 
-class FakeBtrfsUtilsTestCase(BtrfsTestCase):
-    # no setUp nor tearDown needed, we are gonna use fake utils
-    def setUp(self):
-        pass
-
+class FakeBtrfsUtilsTestCase(BtrfsTest):
     @tag_test(TestTags.NOSTORAGE)
     def test_list_subvols_weird_docker_data(self):
         """Verify that list_subvolumes works as expected on weird data from one Docker use case"""
 
-        with fake_utils("tests/btrfs_subvols_docker"):
+        BlockDev.btrfs_is_tech_avail(BlockDev.BtrfsTech.FS, 0)
+
+        with fake_utils("tests/fake_utils/btrfs_subvols_docker"):
             subvols = BlockDev.btrfs_list_subvolumes("fake_dev", False)
 
         # make sure subvolumes are sorted properly (parents before children)
@@ -565,60 +558,23 @@ class FakeBtrfsUtilsTestCase(BtrfsTestCase):
         # check that one of the weird subvols is in the list of subvolumes
         self.assertTrue(any(subvol for subvol in subvols if subvol.path == "docker/btrfs/subvolumes/f2062b736fbabbe4da752632ac4deae87fcb916add6d7d8f5cecee4cbdc41fd9"))
 
-class BTRFSUnloadTest(BtrfsTestCase):
-    def setUp(self):
-        # make sure the library is initialized with all plugins loaded for other
-        # tests
-        self.addCleanup(BlockDev.reinit, self.requested_plugins, True, None)
-
+class BTRFSSkipTest(BtrfsTest):
     @tag_test(TestTags.NOSTORAGE)
-    def test_check_low_version(self):
-        """Verify that checking the minimum BTRFS version works as expected"""
+    def test_missing_dependencies(self):
+        """Verify that checking for technology support works as expected"""
 
-        # unload all plugins first
-        self.assertTrue(BlockDev.reinit([], True, None))
-
-        with fake_utils("tests/btrfs_low_version/"):
-            # too low version of BTRFS available, the BTRFS plugin should fail to load
-            with self.assertRaises(GLib.GError):
-                BlockDev.reinit(self.requested_plugins, True, None)
-
-            self.assertNotIn("btrfs", BlockDev.get_available_plugin_names())
-
-        # load the plugins back
-        self.assertTrue(BlockDev.reinit(self.requested_plugins, True, None))
-        self.assertIn("btrfs", BlockDev.get_available_plugin_names())
-
-    @tag_test(TestTags.NOSTORAGE)
-    def test_check_new_version_format(self):
-        """Verify that checking the minimum BTRFS version works as expected with the new format"""
-
-        # unload all plugins first
-        self.assertTrue(BlockDev.reinit([], True, None))
-
-        # check that new version format is correctly parsed
-        with fake_utils("tests/btrfs_new_version_format/"):
-            BlockDev.reinit(self.requested_plugins, True, None)
-
-        self.assertIn("btrfs", BlockDev.get_available_plugin_names())
-
-        BlockDev.reinit(self.requested_plugins, True, None)
-        self.assertIn("btrfs", BlockDev.get_available_plugin_names())
-
-    @tag_test(TestTags.NOSTORAGE)
-    def test_check_no_btrfs(self):
-        """Verify that checking btrfs tool availability works as expected"""
-
-        # unload all plugins first
-        self.assertTrue(BlockDev.reinit([], True, None))
+        with fake_utils("tests/fake_utils/btrfs_low_version/"):
+            # too low version of BTRFS available
+            with self.assertRaisesRegex(GLib.GError, "Too low version of btrfs"):
+                BlockDev.btrfs_is_tech_avail(BlockDev.BtrfsTech.FS, 0)
 
         with fake_path(all_but="btrfs"):
-            # no btrfs tool available, the BTRFS plugin should fail to load
-            with self.assertRaises(GLib.GError):
-                BlockDev.reinit(self.requested_plugins, True, None)
+            # no btrfs tool available
+            with self.assertRaisesRegex(GLib.GError, "The 'btrfs' utility is not available"):
+                BlockDev.btrfs_is_tech_avail(BlockDev.BtrfsTech.FS, 0)
 
-            self.assertNotIn("btrfs", BlockDev.get_available_plugin_names())
+        # check that new version format is correctly parsed
+        with fake_utils("tests/fake_utils/btrfs_new_version_format/"):
+            avail = BlockDev.btrfs_is_tech_avail(BlockDev.BtrfsTech.FS, 0)
+            self.assertTrue(avail)
 
-        # load the plugins back
-        self.assertTrue(BlockDev.reinit(self.requested_plugins, True, None))
-        self.assertIn("btrfs", BlockDev.get_available_plugin_names())
